@@ -113,9 +113,6 @@ public:
         /// Normal in view space
         SPC_NORMAL_VIEW_SPACE,
 
-        /// Normal in tangent space
-        SPC_NORMAL_TANGENT_SPACE,
-
         /// View vector in object space
         SPC_POSTOCAMERA_OBJECT_SPACE,
 
@@ -125,122 +122,6 @@ public:
         /// View vector in view space
         SPC_POSTOCAMERA_VIEW_SPACE,
 
-        /// View vector in tangent space
-        SPC_POSTOCAMERA_TANGENT_SPACE,
-
-        /// Light vector in object space index 0-7
-        SPC_POSTOLIGHT_OBJECT_SPACE0,
-        SPC_POSTOLIGHT_OBJECT_SPACE1,
-        SPC_POSTOLIGHT_OBJECT_SPACE2,
-        SPC_POSTOLIGHT_OBJECT_SPACE3,
-        SPC_POSTOLIGHT_OBJECT_SPACE4,
-        SPC_POSTOLIGHT_OBJECT_SPACE5,
-        SPC_POSTOLIGHT_OBJECT_SPACE6,
-        SPC_POSTOLIGHT_OBJECT_SPACE7,
-
-        /// Light vector in world space index 0-7
-        SPC_POSTOLIGHT_WORLD_SPACE0,
-        SPC_POSTOLIGHT_WORLD_SPACE1,
-        SPC_POSTOLIGHT_WORLD_SPACE2,
-        SPC_POSTOLIGHT_WORLD_SPACE3,
-        SPC_POSTOLIGHT_WORLD_SPACE4,
-        SPC_POSTOLIGHT_WORLD_SPACE5,
-        SPC_POSTOLIGHT_WORLD_SPACE6,
-        SPC_POSTOLIGHT_WORLD_SPACE7,
-
-        /// Light vector in view space index 0-7
-        SPC_POSTOLIGHT_VIEW_SPACE0,
-        SPC_POSTOLIGHT_VIEW_SPACE1,
-        SPC_POSTOLIGHT_VIEW_SPACE2,
-        SPC_POSTOLIGHT_VIEW_SPACE3,
-        SPC_POSTOLIGHT_VIEW_SPACE4,
-        SPC_POSTOLIGHT_VIEW_SPACE5,
-        SPC_POSTOLIGHT_VIEW_SPACE6,
-        SPC_POSTOLIGHT_VIEW_SPACE7,
-
-        /// Light vector in tangent space index 0-7
-        SPC_POSTOLIGHT_TANGENT_SPACE0,
-        SPC_POSTOLIGHT_TANGENT_SPACE1,
-        SPC_POSTOLIGHT_TANGENT_SPACE2,
-        SPC_POSTOLIGHT_TANGENT_SPACE3,
-        SPC_POSTOLIGHT_TANGENT_SPACE4,
-        SPC_POSTOLIGHT_TANGENT_SPACE5,
-        SPC_POSTOLIGHT_TANGENT_SPACE6,
-        SPC_POSTOLIGHT_TANGENT_SPACE7,
-
-        /// Light direction in object space index 0-7
-        SPC_LIGHTDIRECTION_OBJECT_SPACE0,
-        SPC_LIGHTDIRECTION_OBJECT_SPACE1,
-        SPC_LIGHTDIRECTION_OBJECT_SPACE2,
-        SPC_LIGHTDIRECTION_OBJECT_SPACE3,
-        SPC_LIGHTDIRECTION_OBJECT_SPACE4,
-        SPC_LIGHTDIRECTION_OBJECT_SPACE5,
-        SPC_LIGHTDIRECTION_OBJECT_SPACE6,
-        SPC_LIGHTDIRECTION_OBJECT_SPACE7,
-
-        /// Light direction in world space index 0-7
-        SPC_LIGHTDIRECTION_WORLD_SPACE0,
-        SPC_LIGHTDIRECTION_WORLD_SPACE1,
-        SPC_LIGHTDIRECTION_WORLD_SPACE2,
-        SPC_LIGHTDIRECTION_WORLD_SPACE3,
-        SPC_LIGHTDIRECTION_WORLD_SPACE4,
-        SPC_LIGHTDIRECTION_WORLD_SPACE5,
-        SPC_LIGHTDIRECTION_WORLD_SPACE6,
-        SPC_LIGHTDIRECTION_WORLD_SPACE7,
-
-        /// Light direction in view space index 0-7
-        SPC_LIGHTDIRECTION_VIEW_SPACE0,
-        SPC_LIGHTDIRECTION_VIEW_SPACE1,
-        SPC_LIGHTDIRECTION_VIEW_SPACE2,
-        SPC_LIGHTDIRECTION_VIEW_SPACE3,
-        SPC_LIGHTDIRECTION_VIEW_SPACE4,
-        SPC_LIGHTDIRECTION_VIEW_SPACE5,
-        SPC_LIGHTDIRECTION_VIEW_SPACE6,
-        SPC_LIGHTDIRECTION_VIEW_SPACE7,
-
-        /// Light direction in tangent space index 0-7
-        SPC_LIGHTDIRECTION_TANGENT_SPACE0,
-        SPC_LIGHTDIRECTION_TANGENT_SPACE1,
-        SPC_LIGHTDIRECTION_TANGENT_SPACE2,
-        SPC_LIGHTDIRECTION_TANGENT_SPACE3,
-        SPC_LIGHTDIRECTION_TANGENT_SPACE4,
-        SPC_LIGHTDIRECTION_TANGENT_SPACE5,
-        SPC_LIGHTDIRECTION_TANGENT_SPACE6,
-        SPC_LIGHTDIRECTION_TANGENT_SPACE7,
-
-        /// Light position in object space index 0-7
-        SPC_LIGHTPOSITION_OBJECT_SPACE0,
-        SPC_LIGHTPOSITION_OBJECT_SPACE1,
-        SPC_LIGHTPOSITION_OBJECT_SPACE2,
-        SPC_LIGHTPOSITION_OBJECT_SPACE3,
-        SPC_LIGHTPOSITION_OBJECT_SPACE4,
-        SPC_LIGHTPOSITION_OBJECT_SPACE5,
-        SPC_LIGHTPOSITION_OBJECT_SPACE6,
-        SPC_LIGHTPOSITION_OBJECT_SPACE7,
-
-        /// Light position in world space index 0-7
-        SPC_LIGHTPOSITION_WORLD_SPACE0,
-        SPC_LIGHTPOSITION_WORLD_SPACE1,
-        SPC_LIGHTPOSITION_WORLD_SPACE2,
-        SPC_LIGHTPOSITION_WORLD_SPACE3,
-        SPC_LIGHTPOSITION_WORLD_SPACE4,
-        SPC_LIGHTPOSITION_WORLD_SPACE5,
-        SPC_LIGHTPOSITION_WORLD_SPACE6,
-        SPC_LIGHTPOSITION_WORLD_SPACE7,
-
-        /// Light position in view space index 0-7
-        SPC_LIGHTPOSITIONVIEW_SPACE0,
-        SPC_LIGHTPOSITIONVIEW_SPACE1,
-        SPC_LIGHTPOSITIONVIEW_SPACE2,
-        SPC_LIGHTPOSITIONVIEW_SPACE3,
-        SPC_LIGHTPOSITIONVIEW_SPACE4,
-        SPC_LIGHTPOSITIONVIEW_SPACE5,
-        SPC_LIGHTPOSITIONVIEW_SPACE6,
-        SPC_LIGHTPOSITIONVIEW_SPACE7,
-
-        /// Light position in tangent space index 0-7
-        SPC_LIGHTPOSITION_TANGENT_SPACE,
-
         /// Blending weights
         SPC_BLEND_WEIGHTS,
 
@@ -256,9 +137,6 @@ public:
         /// Tangent in view space
         SPC_TANGENT_VIEW_SPACE,
 
-        /// Tangent in tangent space
-        SPC_TANGENT_TANGENT_SPACE,
-
         /// Binormal in object space
         SPC_BINORMAL_OBJECT_SPACE,
 
@@ -268,18 +146,12 @@ public:
         /// Binormal in view space
         SPC_BINORMAL_VIEW_SPACE,
 
-        /// Binormal in tangent space
-        SPC_BINORMAL_TANGENT_SPACE,
-
         /// Diffuse color
         SPC_COLOR_DIFFUSE,
 
         /// Specular color
         SPC_COLOR_SPECULAR,
 
-        /// Depth in object space
-        SPC_DEPTH_OBJECT_SPACE,
-
         /// Depth in world space
         SPC_DEPTH_WORLD_SPACE,
 
