@@ -159,29 +159,28 @@ void FunctionStageRef::callBuiltin(const char* name, const std::vector<Operand>&
 }
 
 //-----------------------------------------------------------------------------
+Function::Function(GpuProgramType type) : mType(type) {}
+
 Function::~Function()
 {
-    std::map<size_t, FunctionAtomInstanceList>::iterator jt;
-    for(jt = mAtomInstances.begin(); jt != mAtomInstances.end(); ++jt)
+    for(const auto& j : mAtomInstances)
     {
-        for (FunctionAtomInstanceIterator it=jt->second.begin(); it != jt->second.end(); ++it)
-            OGRE_DELETE (*it);
+        for (auto& i : j.second)
+            OGRE_DELETE i;
     }
-
     mAtomInstances.clear();
 
-    for (ShaderParameterIterator it = mInputParameters.begin(); it != mInputParameters.end(); ++it)
-        (*it).reset();
+    for (auto& p : mInputParameters)
+        p.reset();
     mInputParameters.clear();
 
-    for (ShaderParameterIterator it = mOutputParameters.begin(); it != mOutputParameters.end(); ++it)
-        (*it).reset();
+    for (auto& p : mOutputParameters)
+        p.reset();
     mOutputParameters.clear();
 
-    for (ShaderParameterIterator it = mLocalParameters.begin(); it != mLocalParameters.end(); ++it)
-        (*it).reset();
+    for (auto& p : mLocalParameters)
+        p.reset();
     mLocalParameters.clear();
-
 }
 
 static String getParameterName(const char* prefix, Parameter::Semantic semantic, int index)
@@ -251,14 +250,10 @@ ParameterPtr Function::resolveInputParameter(Parameter::Semantic semantic,
         index = 0;
 
         // Find the next available index of the target semantic.
-        ShaderParameterIterator it;
-
-        for (it = mInputParameters.begin(); it != mInputParameters.end(); ++it)
+        for (auto& p : mInputParameters)
         {
-            if ((*it)->getSemantic() == semantic)
-            {
+            if (p->getSemantic() == semantic)
                 index++;
-            }
         }
     }
     else
@@ -277,11 +272,12 @@ ParameterPtr Function::resolveInputParameter(Parameter::Semantic semantic,
         }
     }
 
-    
+    const char* prefix = mType == GPT_VERTEX_PROGRAM ? "i" : "vs";
+
     // No parameter found -> create new one.
     OgreAssert(semantic != Parameter::SPS_UNKNOWN, "unknown semantic");
     param =
-        std::make_shared<Parameter>(type, getParameterName("i", semantic, index), semantic, index, content);
+        std::make_shared<Parameter>(type, getParameterName(prefix, semantic, index), semantic, index, content);
     addInputParameter(param);
 
     return param;
@@ -290,7 +286,7 @@ ParameterPtr Function::resolveInputParameter(Parameter::Semantic semantic,
 //-----------------------------------------------------------------------------
 ParameterPtr Function::resolveOutputParameter(Parameter::Semantic semantic,
                                             int index,
-                                            Parameter::Content content,                                         
+                                            Parameter::Content content,
                                             GpuConstantType type)
 {
     if(type == GCT_UNKNOWN)
@@ -315,14 +311,10 @@ ParameterPtr Function::resolveOutputParameter(Parameter::Semantic semantic,
         index = 0;
 
         // Find the next available index of the target semantic.
-        ShaderParameterIterator it;
-
-        for (it = mOutputParameters.begin(); it != mOutputParameters.end(); ++it)
+        for (auto& p : mOutputParameters)
         {
-            if ((*it)->getSemantic() == semantic)
-            {
+            if (p->getSemantic() == semantic)
                 index++;
-            }
         }
     }
     else
@@ -340,7 +332,8 @@ ParameterPtr Function::resolveOutputParameter(Parameter::Semantic semantic,
                             semantic, index));
         }
     }
-    
+
+    const char* prefix = mType == GPT_VERTEX_PROGRAM ? "vs" : "o";
 
     // No parameter found -> create new one.
     switch (semantic)
@@ -348,7 +341,7 @@ ParameterPtr Function::resolveOutputParameter(Parameter::Semantic semantic,
     case Parameter::SPS_TEXTURE_COORDINATES:
     case Parameter::SPS_COLOR:
     case Parameter::SPS_POSITION:
-        param = std::make_shared<Parameter>(type, getParameterName("o", semantic, index), semantic, index,
+        param = std::make_shared<Parameter>(type, getParameterName(prefix, semantic, index), semantic, index,
                                             content);
         break;
 
@@ -364,27 +357,27 @@ ParameterPtr Function::resolveOutputParameter(Parameter::Semantic semantic,
 }
 
 //-----------------------------------------------------------------------------
-ParameterPtr Function::resolveLocalParameter(GpuConstantType type, const String& name)
+ParameterPtr Function::resolveLocalParameter(GpuConstantType type, const String& name, size_t arraySize)
 {
     ParameterPtr param;
 
     param = _getParameterByName(mLocalParameters, name);
     if (param.get() != NULL)
     {
-        if (param->getType() == type)
+        if (param->getType() == type && param->getSize() == arraySize)
         {
             return param;
         }
-        else 
+        else
         {
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
                         "Can not resolve local parameter due to type mismatch");
-        }       
+        }
     }
-        
-    param = std::make_shared<Parameter>(type, name, Parameter::SPS_UNKNOWN, 0, Parameter::SPC_UNKNOWN);
+
+    param = std::make_shared<Parameter>(type, name, Parameter::SPS_UNKNOWN, 0, Parameter::SPC_UNKNOWN, arraySize);
     addParameter(mLocalParameters, param);
-            
+
     return param;
 }
 
@@ -414,7 +407,7 @@ ParameterPtr Function::resolveLocalParameter(const Parameter::Content content, G
     if(type == GCT_UNKNOWN) type = typeFromContent(content);
 
     param = _getParameterByContent(mLocalParameters, content, type);
-    if (param.get() != NULL)    
+    if (param.get() != NULL)
         return param;
 
     param = std::make_shared<Parameter>(
@@ -432,7 +425,7 @@ void Function::addInputParameter(ParameterPtr parameter)
     // Check that parameter with the same semantic and index in input parameters list.
     if (_getParameterBySemantic(mInputParameters, parameter->getSemantic(), parameter->getIndex()).get() != NULL)
     {
-        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS, 
+        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS,
             "Parameter <" + parameter->getName() + "> has equal semantic parameter");
     }
 
@@ -445,7 +438,7 @@ void Function::addOutputParameter(ParameterPtr parameter)
     // Check that parameter with the same semantic and index in output parameters list.
     if (_getParameterBySemantic(mOutputParameters, parameter->getSemantic(), parameter->getIndex()).get() != NULL)
     {
-        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS, 
+        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS,
             "Parameter <" + parameter->getName() + "> has equal semantic parameter");
     }
 
@@ -477,19 +470,19 @@ void Function::deleteAllOutputParameters()
 }
 //-----------------------------------------------------------------------------
 void Function::addParameter(ShaderParameterList& parameterList, ParameterPtr parameter)
-                                        
+
 {
     // Check that parameter with the same name doest exist in input parameters list.
     if (_getParameterByName(mInputParameters, parameter->getName()).get() != NULL)
     {
-        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS, 
+        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS,
             "Parameter <" + parameter->getName() + "> already declared");
     }
 
     // Check that parameter with the same name doest exist in output parameters list.
     if (_getParameterByName(mOutputParameters, parameter->getName()).get() != NULL)
     {
-        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS, 
+        OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS,
             "Parameter <" + parameter->getName() + "> already declared");
     }
 
@@ -517,14 +510,10 @@ void Function::deleteParameter(ShaderParameterList& parameterList, ParameterPtr
 //-----------------------------------------------------------------------------
 ParameterPtr Function::_getParameterByName( const ShaderParameterList& parameterList, const String& name )
 {
-    ShaderParameterConstIterator it;
-
-    for (it = parameterList.begin(); it != parameterList.end(); ++it)
+    for (const auto& p : parameterList)
     {
-        if ((*it)->getName() == name)
-        {
-            return *it;
-        }
+        if (p->getName() == name)
+            return p;
     }
 
     return ParameterPtr();
@@ -532,18 +521,13 @@ ParameterPtr Function::_getParameterByName( const ShaderParameterList& parameter
 
 //-----------------------------------------------------------------------------
 ParameterPtr Function::_getParameterBySemantic(const ShaderParameterList& parameterList,
-                                                const Parameter::Semantic semantic, 
+                                                const Parameter::Semantic semantic,
                                                 int index)
 {
-    ShaderParameterConstIterator it;
-
-    for (it = parameterList.begin(); it != parameterList.end(); ++it)
+    for (const auto& p : parameterList)
     {
-        if ((*it)->getSemantic() == semantic &&
-            (*it)->getIndex() == index)
-        {
-            return *it;
-        }
+        if (p->getSemantic() == semantic && p->getIndex() == index)
+            return p;
     }
 
     return ParameterPtr();
@@ -552,24 +536,19 @@ ParameterPtr Function::_getParameterBySemantic(const ShaderParameterList& parame
 //-----------------------------------------------------------------------------
 ParameterPtr Function::_getParameterByContent(const ShaderParameterList& parameterList, const Parameter::Content content, GpuConstantType type)
 {
-    ShaderParameterConstIterator it;
-
     if(type == GCT_UNKNOWN)
         type = typeFromContent(content);
 
     // Search only for known content.
-    if (content != Parameter::SPC_UNKNOWN)  
+    if (content != Parameter::SPC_UNKNOWN)
     {
-        for (it = parameterList.begin(); it != parameterList.end(); ++it)
+        for (const auto& p : parameterList)
         {
-            if ((*it)->getContent() == content &&
-                (*it)->getType() == type)
-            {
-                return *it;
-            }
+            if (p->getContent() == content && p->getType() == type)
+                return p;
         }
     }
-    
+
     return ParameterPtr();
 }
 
@@ -594,11 +573,11 @@ bool Function::deleteAtomInstance(FunctionAtom* atomInstance)
             mAtomInstances[g].erase(it);
             mSortedAtomInstances.clear();
             return true;
-        }       
+        }
     }
 
     return false;
-    
+
 }
 
 //-----------------------------------------------------------------------------
@@ -608,11 +587,9 @@ const FunctionAtomInstanceList& Function::getAtomInstances()
         return mSortedAtomInstances;
 
     // put atom instances into order
-    std::map<size_t, FunctionAtomInstanceList>::const_iterator it;
-    for(it = mAtomInstances.begin(); it != mAtomInstances.end(); ++it)
+    for(const auto& i : mAtomInstances)
     {
-        mSortedAtomInstances.insert(mSortedAtomInstances.end(), it->second.begin(),
-                                    it->second.end());
+        mSortedAtomInstances.insert(mSortedAtomInstances.end(), i.second.begin(), i.second.end());
     }
 
     return mSortedAtomInstances;
