@@ -31,8 +31,6 @@ THE SOFTWARE.
 
 #include "OgrePrerequisites.h"
 #include "OgreHardwareBufferManager.h"
-#include "OgreHardwareIndexBuffer.h"
-#include "OgreHardwareVertexBuffer.h"
 
 namespace Ogre {
     /** \addtogroup Core
@@ -54,30 +52,11 @@ namespace Ogre {
         ~DefaultHardwareBuffer();
         void readData(size_t offset, size_t length, void* pDest) override;
         void writeData(size_t offset, size_t length, const void* pSource, bool discardWholeBuffer = false) override;
+        bool isSystemMemory(void) const override { return true; }
     };
 
     typedef DefaultHardwareBuffer DefaultHardwareUniformBuffer;
 
-    class _OgreExport DefaultHardwareVertexBuffer : public HardwareVertexBuffer
-    {
-    public:
-        DefaultHardwareVertexBuffer(size_t vertexSize, size_t numVertices, Usage = HBU_CPU_ONLY)
-            : HardwareVertexBuffer(NULL, vertexSize, numVertices,
-                                   new DefaultHardwareBuffer(vertexSize * numVertices))
-        {
-        }
-    };
-
-    class _OgreExport DefaultHardwareIndexBuffer : public HardwareIndexBuffer
-    {
-    public:
-        DefaultHardwareIndexBuffer(IndexType idxType, size_t numIndexes, Usage = HBU_CPU_ONLY)
-            : HardwareIndexBuffer(NULL, idxType, numIndexes,
-                                  new DefaultHardwareBuffer(indexSize(idxType) * numIndexes))
-        {
-        }
-    };
-
     /** Specialisation of HardwareBufferManagerBase to emulate hardware buffers.
 
         You might want to instantiate this class if you want to utilise
