@@ -42,20 +42,6 @@ namespace Ogre
     /** \addtogroup Scene
     *  @{
     */
-    /** Specifies orientation mode.
-    */
-    enum OrientationMode : uint8
-    {
-        OR_DEGREE_0       = 0,
-        OR_DEGREE_90      = 1,
-        OR_DEGREE_180     = 2,
-        OR_DEGREE_270     = 3,
-
-        OR_PORTRAIT       = OR_DEGREE_0,
-        OR_LANDSCAPERIGHT = OR_DEGREE_90,
-        OR_LANDSCAPELEFT  = OR_DEGREE_270
-    };
-
     /** Specifies perspective (realistic) or orthographic (architectural) projection.
     */
     enum ProjectionType : uint8
@@ -86,9 +72,9 @@ namespace Ogre
         /// y-direction field-of-view (default 45)
         Radian mFOVy;
         /// Far clip distance - default 10000
-        Real mFarDist;
+        float mFarDist;
         /// Near clip distance - default 100
-        Real mNearDist;
+        float mNearDist;
         /// x/y viewport ratio - default 1.3333
         Real mAspect;
         /// Ortho height size (world units)
@@ -105,8 +91,6 @@ namespace Ogre
         mutable Quaternion mLastParentOrientation;
         mutable Vector3 mLastParentPosition;
 
-        /// Pre-calced projection matrix for the specific render system
-        mutable Matrix4 mProjMatrixRS;
         /// Pre-calced standard projection matrix but with render system depth range
         mutable Matrix4 mProjMatrixRSDepth;
         /// Pre-calced standard projection matrix
@@ -155,9 +139,6 @@ namespace Ogre
         /// Signal to update view information.
         virtual void invalidateView(void) const;
 
-        /// Shared class-level name for Movable type
-        static String msMovableType;
-
         ColourValue mDebugColour;
         /// Pointer to a reflection plane (automatically updated)
         const MovablePlane* mLinkedReflectPlane;
@@ -182,8 +163,6 @@ namespace Ogre
         bool mReflect;
         /// Is this frustum using an oblique depth projection?
         bool mObliqueDepthProjection;
-        /// Frustum orientation mode
-        mutable OrientationMode mOrientationMode;
 
     public:
 
@@ -218,11 +197,11 @@ namespace Ogre
         @param nearDist
             The distance to the near clipping plane from the frustum in world coordinates.
          */
-        void setNearClipDistance(Real nearDist);
+        void setNearClipDistance(float nearDist);
 
         /** Retrieves the distance from the frustum to the near clipping plane.
         */
-        Real getNearClipDistance(void) const;
+        virtual float getNearClipDistance(void) const { return mNearDist; }
 
         /** Sets the distance to the far clipping plane.
 
@@ -244,11 +223,11 @@ namespace Ogre
             distance which is useful especially when projecting shadows; but
             be careful not to use a near distance too close.
         */
-        void setFarClipDistance(Real farDist);
+        void setFarClipDistance(float farDist);
 
         /** Retrieves the distance from the frustum to the far clipping plane.
         */
-        Real getFarClipDistance(void) const;
+        virtual float getFarClipDistance(void) const { return mFarDist; }
 
         /** Sets the aspect ratio for the frustum viewport.
 
@@ -316,13 +295,6 @@ namespace Ogre
         /** Get the extents of the frustum in view space. */
         RealRect getFrustumExtents() const;
 
-        /** Gets the projection matrix for this frustum adjusted for the current
-            rendersystem specifics (may be right or left-handed, depth range
-            may vary).
-
-            @deprecated do not use
-        */
-        OGRE_DEPRECATED const Matrix4& getProjectionMatrixRS(void) const;
         /** Gets the depth-adjusted projection matrix for the current rendersystem
 
             This differs from the rendering-API independent @ref getProjectionMatrix
@@ -339,7 +311,7 @@ namespace Ogre
 
         /** Gets the view matrix for this frustum. Mainly for use by OGRE internally.
         */
-        const Affine3& getViewMatrix(void) const;
+        virtual const Affine3& getViewMatrix(void) const;
 
         /** Calculate a view matrix for this frustum, relative to a potentially dynamic point. 
             Mainly for use by OGRE internally when using camera-relative rendering
@@ -624,19 +596,6 @@ namespace Ogre
         /** Gets a world-space list of planes enclosing the frustum.
         */
         PlaneBoundedVolume getPlaneBoundedVolume();
-        /** Set the orientation mode of the frustum. Default is OR_DEGREE_0
-
-            Setting the orientation of a frustum is only supported on
-            iOS at this time.  An exception is thrown on other platforms.
-        */
-        void setOrientationMode(OrientationMode orientationMode);
-
-        /** Get the orientation mode of the frustum.
-
-            Getting the orientation of a frustum is only supported on
-            iOS at this time.  An exception is thrown on other platforms.
-        */
-        OrientationMode getOrientationMode() const;
 
     };
 
