@@ -45,35 +45,33 @@ namespace Ogre {
     *  @{
     */
     /** Specialisation of HardwareBuffer for a vertex buffer. */
-    class _OgreExport HardwareVertexBuffer : public HardwareBuffer
+    class _OgreExport HardwareVertexBuffer final : public HardwareBuffer
     {
             bool mIsInstanceData;
             HardwareBufferManagerBase* mMgr;
-            size_t mNumVertices;
-            size_t mVertexSize;
-            size_t mInstanceDataStepRate;           
-            /// Checks if vertex instance data is supported by the render system
-            virtual bool checkIfVertexInstanceDataIsSupported();
+            uint32 mNumVertices;
+            uint32 mVertexSize;
+            uint32 mInstanceDataStepRate;
 
         public:
             /// Should be called by HardwareBufferManager
             HardwareVertexBuffer(HardwareBufferManagerBase* mgr, size_t vertexSize, size_t numVertices,
-                                 Usage usage, bool useSystemMemory, bool useShadowBuffer);
+                                 Usage usage, bool useShadowBuffer);
             HardwareVertexBuffer(HardwareBufferManagerBase* mgr, size_t vertexSize, size_t numVertices,
                                  HardwareBuffer* delegate);
             ~HardwareVertexBuffer();
             /// Return the manager of this buffer, if any
             HardwareBufferManagerBase* getManager() const { return mMgr; }
             /// Gets the size in bytes of a single vertex in this buffer
-            size_t getVertexSize(void) const { return mVertexSize; }
+            uint32 getVertexSize(void) const { return mVertexSize; }
             /// Get the number of vertices in this buffer
-            size_t getNumVertices(void) const { return mNumVertices; }
+            uint32 getNumVertices(void) const { return mNumVertices; }
             /// Get if this vertex buffer is an "instance data" buffer (per instance)
             bool isInstanceData() const { return mIsInstanceData; }
             /// Set if this vertex buffer is an "instance data" buffer (per instance)
             void setIsInstanceData(const bool val);
             /// Get the number of instances to draw using the same per-instance data before advancing in the buffer by one element.
-            size_t getInstanceDataStepRate() const;
+            uint32 getInstanceDataStepRate() const;
             /// Set the number of instances to draw using the same per-instance data before advancing in the buffer by one element.
             void setInstanceDataStepRate(const size_t val);
 
@@ -86,7 +84,8 @@ namespace Ogre {
     OGRE_DEPRECATED typedef HardwareBufferLockGuard HardwareVertexBufferLockGuard;
 
     /// Vertex element semantics, used to identify the meaning of vertex buffer contents
-    enum VertexElementSemantic {
+    enum VertexElementSemantic : uint8
+    {
         /// Position, typically VET_FLOAT3
         VES_POSITION = 1,
         /// Blending weights
@@ -116,20 +115,20 @@ namespace Ogre {
     /**
      * Vertex element type, used to identify the base types of the vertex contents
      *
-     * @note VET_SHORT1, VET_SHORT3, VET_USHORT1 and VET_USHORT3 should never be used
-     * because they aren't supported on any known hardware - they are unaligned as their size
-     * is not a multiple of 4 bytes. Therefore drivers usually must add padding on upload.
+     * VET_SHORT1, VET_SHORT3, VET_USHORT1 and VET_USHORT3 should be used with caution
+     * because they aren't supported on D3D9 and D3D11 - they are unaligned as their size
+     * is not a multiple of 4 bytes. Therefore drivers usually add padding on upload.
      */
-    enum VertexElementType
+    enum VertexElementType : uint8
     {
         VET_FLOAT1 = 0,
         VET_FLOAT2 = 1,
         VET_FLOAT3 = 2,
         VET_FLOAT4 = 3,
 
-        VET_SHORT1 = 5,  ///< @deprecated (see #VertexElementType note)
+        VET_SHORT1 = 5,  ///< not supported on D3D9
         VET_SHORT2 = 6,
-        VET_SHORT3 = 7,  ///< @deprecated (see #VertexElementType note)
+        VET_SHORT3 = 7,  ///< not supported on D3D9 and D3D11
         VET_SHORT4 = 8,
         VET_UBYTE4 = 9,
         _DETAIL_SWAP_RB = 10,
@@ -139,9 +138,9 @@ namespace Ogre {
         VET_DOUBLE2 = 13,
         VET_DOUBLE3 = 14,
         VET_DOUBLE4 = 15,
-        VET_USHORT1 = 16,  ///< @deprecated (see #VertexElementType note)
+        VET_USHORT1 = 16,  ///< not supported on D3D9
         VET_USHORT2 = 17,
-        VET_USHORT3 = 18,  ///< @deprecated (see #VertexElementType note)
+        VET_USHORT3 = 18,  ///< not supported on D3D9 and D3D11
         VET_USHORT4 = 19,
         VET_INT1 = 20,
         VET_INT2 = 21,
@@ -159,6 +158,10 @@ namespace Ogre {
         VET_USHORT2_NORM = 33, ///< unsigned shorts (normalized to 0..1)
         VET_USHORT4_NORM = 34,
         VET_INT_10_10_10_2_NORM = 35, ///< signed int (normalized to 0..1)
+        VET_HALF1 = 36, ///< not supported on D3D9
+        VET_HALF2 = 37,
+        VET_HALF3 = 38, ///< not supported on D3D9 and D3D11
+        VET_HALF4 = 39,
         VET_COLOUR = VET_UBYTE4_NORM,  ///< @deprecated use VET_UBYTE4_NORM
         VET_COLOUR_ARGB = VET_UBYTE4_NORM,  ///< @deprecated use VET_UBYTE4_NORM
         VET_COLOUR_ABGR = VET_UBYTE4_NORM,  ///< @deprecated use VET_UBYTE4_NORM
@@ -276,8 +279,6 @@ namespace Ogre {
     public:
         /// Defines the list of vertex elements that makes up this declaration
         typedef std::list<VertexElement> VertexElementList;
-        /// Sort routine for vertex elements
-        static bool vertexElementLess(const VertexElement& e1, const VertexElement& e2);
     protected:
         VertexElementList mElementList;
 
