@@ -383,15 +383,15 @@ namespace Ogre {
 
         /** Sets the default dimensions of the particles in this set.
 
-                All particles in a set are created with these default dimensions. The set will render most efficiently if
-                all the particles in the set are the default size. It is possible to alter the size of individual
-                particles at the expense of extra calculation. See the Particle class for more info.
+            All particles in a set are created with these default dimensions. The set will render most efficiently if
+            all the particles in the set are the default size. It is possible to alter the size of individual
+            particles at the expense of extra calculation. See the Particle class for more info.
             @param width
                 The new default width for the particles in this set. Must be non-negative!
             @param height
                 The new default height for the particles in this set. Must be non-negative!
         */
-        virtual void setDefaultDimensions(Real width, Real height);
+        void setDefaultDimensions(Real width, Real height);
 
         /** See setDefaultDimensions - this sets 1 component individually. */
         virtual void setDefaultWidth(Real width);
@@ -567,7 +567,7 @@ namespace Ogre {
         /// Last frame in which known to be visible
         unsigned long mLastVisibleFrame;
         /// Controller for time update
-        ControllerReal* mTimeController;
+        ControllerFloat* mTimeController;
         /// Indication whether the emitted emitter pool (= pool with particle emitters that are emitted) is initialised
         bool mEmittedEmitterPoolInitialised;
         /// Used to control if the particle system should emit particles or not.
