@@ -66,9 +66,10 @@ namespace Ogre {
         RENDER_QUEUE_7 = 70,
         RENDER_QUEUE_WORLD_GEOMETRY_2 = 75,
         RENDER_QUEUE_8 = 80,
-        RENDER_QUEUE_9 = 90,
-        /// Penultimate queue(before overlays), used for skyboxes if rendered last
-        RENDER_QUEUE_SKIES_LATE = 95,
+        /// Used for skyboxes if rendered last
+        RENDER_QUEUE_SKIES_LATE = 90,
+        /// Penultimate queue(before overlays)
+        RENDER_QUEUE_TRANSPARENTS = 95,
         /// Use this queue for objects which must be rendered last e.g. overlays
         RENDER_QUEUE_OVERLAY = 100, 
         /// Final possible render queue, don't exceed this
@@ -81,8 +82,8 @@ namespace Ogre {
 
     /** Class to manage the scene object rendering queue.
 
-            Objects are grouped by material to minimise rendering state changes. The map from
-            material to renderable object is wrapped in a class for ease of use.
+        Objects are grouped by material to minimise rendering state changes. The map from
+        material to renderable object is wrapped in a class for ease of use.
         @par
             This class now includes the concept of 'queue groups' which allows the application
             adding the renderable to specifically schedule it so that it is included in 
