@@ -336,20 +336,8 @@ namespace Ogre {
 
         /// @deprecated use StringUtil::split
         OGRE_DEPRECATED static StringVector parseStringVector(const String& val) { return StringUtil::split(val); }
-        /** Checks the String is a valid number value. */
-        static bool isNumber(const String& val);
-
-		/** Converts a StereoModeType to a String
-
-			String output format is "None", "Frame Sequential", etc.
-		*/
-		static String toString(StereoModeType val);
-
-		/** Converts a String to a StereoModeType
-
-			String input format should be "None", "Frame Sequential", etc.
-		*/
-		static StereoModeType parseStereoMode(const String& val, StereoModeType defaultValue = SMT_NONE);
+        /// @deprecated use @ref parse()
+        OGRE_DEPRECATED static bool isNumber(const String& val);
 
 		static locale_t _numLocale;
     private:
