@@ -29,7 +29,6 @@ THE SOFTWARE.
 
 #include "OgreAutoParamDataSource.h"
 #include "OgreRenderable.h"
-#include "OgreRenderTarget.h"
 #include "OgreControllerManager.h"
 #include "OgreViewport.h"
 
@@ -214,10 +213,16 @@ namespace Ogre {
         return getLight(index).getPowerScale();
     }
     //-----------------------------------------------------------------------------
-    const Vector4f& AutoParamDataSource::getLightAttenuation(size_t index) const
+    Vector4f AutoParamDataSource::getLightAttenuation(size_t index) const
     {
+        const Light& l = getLight(index);
+        if(l.getType() == Light::LT_RECTLIGHT)
+        {
+            auto rot = getViewMatrix().linear();
+            return Vector4f(Vector3f(rot * Vector3(l.getDerivedSourceHalfHeight())), 0.0);
+        }
         // range, const, linear, quad
-        return getLight(index).getAttenuation();
+        return l.getAttenuation();
     }
     //-----------------------------------------------------------------------------
     Vector4f AutoParamDataSource::getSpotlightParams(size_t index) const
@@ -231,6 +236,11 @@ namespace Ogre {
                            l.getSpotlightFalloff(),
                            1.0);
         }
+        else if(l.getType() == Light::LT_RECTLIGHT)
+        {
+            auto rot = getViewMatrix().linear();
+            return Vector4f(Vector3f(rot * Vector3(l.getDerivedSourceHalfWidth())), 2.0);
+        }
         else
         {
             // Use safe values which result in no change to point & dir light calcs
@@ -682,7 +692,7 @@ namespace Ogre {
                     mTextureViewProjMatrix[index] = 
                         Matrix4::CLIPSPACE2DTOIMAGESPACE *
                         mCurrentTextureProjector[index]->getProjectionMatrixWithRSDepth() * 
-                        mCurrentTextureProjector[index]->getViewMatrix();
+                        mCurrentTextureProjector[index]->Frustum::getViewMatrix();
                 }
                 mTextureViewProjMatrixDirty[index] = false;
             }
@@ -1024,17 +1034,17 @@ namespace Ogre {
         return mCurrentCamera->getDerivedUp();
     }
     //-----------------------------------------------------------------------------
-    Real AutoParamDataSource::getFOV() const
+    float AutoParamDataSource::getFOV() const
     { 
         return mCurrentCamera->getFOVy().valueRadians(); 
     }
     //-----------------------------------------------------------------------------
-    Real AutoParamDataSource::getNearClipDistance() const
+    float AutoParamDataSource::getNearClipDistance() const
     { 
         return mCurrentCamera->getNearClipDistance(); 
     }
     //-----------------------------------------------------------------------------
-    Real AutoParamDataSource::getFarClipDistance() const
+    float AutoParamDataSource::getFarClipDistance() const
     { 
         return mCurrentCamera->getFarClipDistance(); 
     }
@@ -1053,6 +1063,10 @@ namespace Ogre {
     {
         ++mPassNumber;
     }
+    int AutoParamDataSource::getMaterialLodIndex() const
+    {
+        return mCurrentRenderable->_getMaterialLodIndex();
+    }
     //-----------------------------------------------------------------------------
     const Vector4& AutoParamDataSource::getSceneDepthRange() const
     {
