@@ -32,8 +32,6 @@ THE SOFTWARE.
 #include "OgreCompositionPass.h"
 #include "OgreCompositionTargetPass.h"
 #include "OgreCompositionTechnique.h"
-#include "OgreRectangle2D.h"
-#include "OgreRenderTarget.h"
 
 namespace Ogre {
 
@@ -49,8 +47,6 @@ CompositorManager& CompositorManager::getSingleton(void)
 CompositorManager::CompositorManager():
     mRectangle(0)
 {
-    initialise();
-
     // Loading order (just after materials)
     mLoadOrder = 110.0f;
 
@@ -93,10 +89,6 @@ CompositorPtr CompositorManager::getByName(const String& name, const String& gro
     return static_pointer_cast<Compositor>(getResourceByName(name, groupName));
 }
 //-----------------------------------------------------------------------
-void CompositorManager::initialise(void)
-{
-}
-//-----------------------------------------------------------------------
 CompositorChain *CompositorManager::getCompositorChain(Viewport *vp)
 {
     Chains::iterator i=mChains.find(vp);
@@ -328,7 +320,7 @@ bool CompositorManager::isInputPreviousTarget(CompositorInstance* inst, const Og
 
 }
 //---------------------------------------------------------------------
-bool CompositorManager::isInputPreviousTarget(CompositorInstance* inst, TexturePtr tex)
+bool CompositorManager::isInputPreviousTarget(CompositorInstance* inst, const TexturePtr& tex)
 {
     const CompositionTechnique::TargetPasses& passes = inst->getTechnique()->getTargetPasses();
     for (auto *tp : passes)
@@ -363,7 +355,7 @@ bool CompositorManager::isInputToOutputTarget(CompositorInstance* inst, const Og
 
 }
 //---------------------------------------------------------------------()
-bool CompositorManager::isInputToOutputTarget(CompositorInstance* inst, TexturePtr tex)
+bool CompositorManager::isInputToOutputTarget(CompositorInstance* inst, const TexturePtr& tex)
 {
     CompositionTargetPass* tp = inst->getTechnique()->getOutputTargetPass();
     for (auto *p : tp->getPasses())
