@@ -1199,13 +1199,5 @@ namespace Ogre
 
         return false; // not found!
     }
-    //-----------------------------------------------------------------------
-    void ConvexBody::logInfo( void ) const
-    {
-        StringStream ssOut( std::stringstream::out );
-        ssOut << *this;
-        
-        Ogre::LogManager::getSingleton().logMessage( Ogre::LML_NORMAL, ssOut.str()  );
-    }
 }
 
