@@ -374,6 +374,9 @@ namespace Ogre
                     case Light::LT_SPOTLIGHT:
                         writeValue("spot");
                         break;
+                    case Light::LT_RECTLIGHT:
+                        writeValue("rect");
+                        break;
                     };
                 }
             }
@@ -384,7 +387,7 @@ namespace Ogre
                 writeValue(StringConverter::toString(pPass->getLightMask()));
             }
 
-            if (pPass->getLightingEnabled())
+            if (pPass->getLightingEnabled() || pPass->getVertexColourTracking() != TVC_NONE)
             {
                 // Ambient
                 if (mDefaults ||
@@ -1056,45 +1059,37 @@ namespace Ogre
             float scrollAnimU = 0;
             float scrollAnimV = 0;
 
-            EffectMap effMap = pTex->getEffects();
-            if (!effMap.empty())
+            for (const auto& it : pTex->getEffects())
             {
-                EffectMap::const_iterator it;
-                for (it = effMap.begin(); it != effMap.end(); ++it)
+                const TextureUnitState::TextureEffect& ef = it.second;
+                switch (ef.type)
                 {
-                    const TextureUnitState::TextureEffect& ef = it->second;
-                    switch (ef.type)
-                    {
-                    case TextureUnitState::ET_ENVIRONMENT_MAP :
-                        writeEnvironmentMapEffect(ef, pTex);
-                        break;
-                    case TextureUnitState::ET_ROTATE :
-                        writeRotationEffect(ef, pTex);
-                        break;
-                    case TextureUnitState::ET_UVSCROLL :
-                        scrollAnimU = scrollAnimV = ef.arg1;
-                        break;
-                    case TextureUnitState::ET_USCROLL :
-                        scrollAnimU = ef.arg1;
-                        break;
-                    case TextureUnitState::ET_VSCROLL :
-                        scrollAnimV = ef.arg1;
-                        break;
-                    case TextureUnitState::ET_TRANSFORM :
-                        writeTransformEffect(ef, pTex);
-                        break;
-                    default:
-                        break;
-                    }
+                case TextureUnitState::ET_ENVIRONMENT_MAP :
+                    writeEnvironmentMapEffect(ef.subtype);
+                    break;
+                case TextureUnitState::ET_ROTATE :
+                    writeRotationEffect(ef.arg1);
+                    break;
+                case TextureUnitState::ET_UVSCROLL :
+                    scrollAnimU = scrollAnimV = ef.arg1;
+                    break;
+                case TextureUnitState::ET_USCROLL :
+                    scrollAnimU = ef.arg1;
+                    break;
+                case TextureUnitState::ET_VSCROLL :
+                    scrollAnimV = ef.arg1;
+                    break;
+                case TextureUnitState::ET_TRANSFORM :
+                    writeTransformEffect(ef, pTex);
+                    break;
+                default:
+                    break;
                 }
             }
 
             // u and v scroll animation speeds merged, if present serialize scroll_anim
             if(scrollAnimU || scrollAnimV) {
-                TextureUnitState::TextureEffect texEffect;
-                texEffect.arg1 = scrollAnimU;
-                texEffect.arg2 = scrollAnimV;
-                writeScrollEffect(texEffect, pTex);
+                writeScrollEffect(scrollAnimU, scrollAnimV);
             }
 
             // Content type
@@ -1129,32 +1124,35 @@ namespace Ogre
 
     }
     //-----------------------------------------------------------------------
-    void MaterialSerializer::writeEnvironmentMapEffect(const TextureUnitState::TextureEffect& effect, const TextureUnitState *pTex)
+    void MaterialSerializer::writeEnvironmentMapEffect(int type)
     {
         writeAttribute(4, "env_map");
-        switch (effect.subtype)
+        switch (type)
         {
-        case TextureUnitState::ENV_PLANAR:
+        case TEXCALC_ENVIRONMENT_MAP_PLANAR:
             writeValue("planar");
             break;
-        case TextureUnitState::ENV_CURVED:
+        case TEXCALC_ENVIRONMENT_MAP:
             writeValue("spherical");
             break;
-        case TextureUnitState::ENV_NORMAL:
+        case TEXCALC_ENVIRONMENT_MAP_NORMAL:
             writeValue("cubic_normal");
             break;
-        case TextureUnitState::ENV_REFLECTION:
+        case TEXCALC_ENVIRONMENT_MAP_REFLECTION:
             writeValue("cubic_reflection");
             break;
+        default:
+            OgreAssert(false, "Unknown environment map type");
+            break;
         }
     }
     //-----------------------------------------------------------------------
-    void MaterialSerializer::writeRotationEffect(const TextureUnitState::TextureEffect& effect, const TextureUnitState *pTex)
+    void MaterialSerializer::writeRotationEffect(float speed)
     {
-        if (effect.arg1)
+        if (speed)
         {
             writeAttribute(4, "rotate_anim");
-            writeValue(StringConverter::toString(effect.arg1));
+            writeValue(StringConverter::toString(speed));
         }
     }
     //-----------------------------------------------------------------------
@@ -1209,15 +1207,11 @@ namespace Ogre
         writeValue(StringConverter::toString(effect.amplitude));
     }
     //-----------------------------------------------------------------------
-    void MaterialSerializer::writeScrollEffect(
-        const TextureUnitState::TextureEffect& effect, const TextureUnitState *pTex)
+    void MaterialSerializer::writeScrollEffect(float scrollAnimU, float scrollAnimV)
     {
-        if (effect.arg1 || effect.arg2)
-        {
-            writeAttribute(4, "scroll_anim");
-            writeValue(StringConverter::toString(effect.arg1));
-            writeValue(StringConverter::toString(effect.arg2));
-        }
+        writeAttribute(4, "scroll_anim");
+        writeValue(StringConverter::toString(scrollAnimU));
+        writeValue(StringConverter::toString(scrollAnimV));
     }
     //-----------------------------------------------------------------------
     void MaterialSerializer::writeSceneBlendFactor(const SceneBlendFactor sbf)
