@@ -4,125 +4,143 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
 
 #include "OgreSkeletonFileFormat.h"
 #include "OgreSkeletonSerializer.h"
-#include "OgreSkeleton.h"
 #include "OgreAnimation.h"
 #include "OgreAnimationTrack.h"
 #include "OgreKeyFrame.h"
-#include "OgreBone.h"
-#include "OgreString.h"
-#include "OgreDataStream.h"
-#include "OgreLogManager.h"
-
-
-
 
 namespace Ogre {
     /// stream overhead = ID + size
-    const long STREAM_OVERHEAD_SIZE = sizeof(uint16) + sizeof(uint32);
+    const long SSTREAM_OVERHEAD_SIZE = sizeof(uint16) + sizeof(uint32);
+    const uint16 HEADER_STREAM_ID_EXT = 0x1000;
     //---------------------------------------------------------------------
     SkeletonSerializer::SkeletonSerializer()
     {
         // Version number
         // NB changed to include bone names in 1.1
-        mVersion = "[Serializer_v1.10]";
+        mVersion = "[Unknown]";
     }
+
     //---------------------------------------------------------------------
-    SkeletonSerializer::~SkeletonSerializer()
+    void SkeletonSerializer::exportSkeleton(const Skeleton* pSkeleton, 
+        const String& filename, SkeletonVersion ver, Endian endianMode)
     {
+        DataStreamPtr stream = _openFileStream(filename, std::ios::binary | std::ios::out);
+        exportSkeleton(pSkeleton, stream, ver, endianMode);
+
+        stream->close();
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::exportSkeleton(const Skeleton* pSkeleton, 
-		const String& filename, Endian endianMode)
+        const DataStreamPtr& stream, SkeletonVersion ver, Endian endianMode)
     {
-		// Decide on endian mode
-		determineEndianness(endianMode);
+        setWorkingVersion(ver);
+        // Decide on endian mode
+        determineEndianness(endianMode);
+
+        mStream = stream; 
+        if (!stream->isWriteable())
+        {
+            OGRE_EXCEPT(Exception::ERR_CANNOT_WRITE_TO_FILE,
+                "Unable to write to stream " + stream->getName(),
+                "SkeletonSerializer::exportSkeleton");
+        }
 
-        String msg;
-        mpfFile = fopen(filename.c_str(), "wb");
-		if (!mpfFile)
-		{
-			OGRE_EXCEPT(Exception::ERR_CANNOT_WRITE_TO_FILE,
-				"Unable to open file " + filename + " for writing",
-				"SkeletonSerializer::exportSkeleton");
-		}
 
         writeFileHeader();
 
+        pushInnerChunk(mStream);
         // Write main skeleton data
         LogManager::getSingleton().logMessage("Exporting bones..");
-        writeSkeleton(pSkeleton);
+        writeSkeleton(pSkeleton, ver);
         LogManager::getSingleton().logMessage("Bones exported.");
 
         // Write all animations
         unsigned short numAnims = pSkeleton->getNumAnimations();
-        msg = "Exporting animations, count=";
-		StringUtil::StrStreamType num;
-		num << numAnims;
-        msg += num.str();
-        LogManager::getSingleton().logMessage(msg);
+        LogManager::getSingleton().stream()
+            << "Exporting animations, count=" << numAnims;
         for (unsigned short i = 0; i < numAnims; ++i)
         {
             Animation* pAnim = pSkeleton->getAnimation(i);
-            msg = "Exporting animation: " + pAnim->getName();
-            LogManager::getSingleton().logMessage(msg);
-            writeAnimation(pSkeleton, pAnim);
+            LogManager::getSingleton().stream()
+                << "Exporting animation: " << pAnim->getName();
+            writeAnimation(pSkeleton, pAnim, ver);
             LogManager::getSingleton().logMessage("Animation exported.");
 
         }
 
-		// Write links
-		Skeleton::LinkedSkeletonAnimSourceIterator linkIt = 
-			pSkeleton->getLinkedSkeletonAnimationSourceIterator();
-		while(linkIt.hasMoreElements())
-		{
-			const LinkedSkeletonAnimationSource& link = linkIt.getNext();
-			writeSkeletonAnimationLink(pSkeleton, link);
-		}
-
-        fclose(mpfFile);
-
+        // Write links
+        for(const auto& link : pSkeleton->getLinkedSkeletonAnimationSources())
+        {
+            writeSkeletonAnimationLink(pSkeleton, link);
+        }       
+        popInnerChunk(stream);
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::importSkeleton(DataStreamPtr& stream, Skeleton* pSkel)
     {
-		// Determine endianness (must be the first thing we do!)
-		determineEndianness(stream);
+        // Determine endianness (must be the first thing we do!)
+        determineEndianness(stream);
 
-		// Check header
-        readFileHeader(stream);
+        // Check header
+        unsigned short headerID;
+        readShorts(stream, &headerID, 1);
+        if (headerID != HEADER_STREAM_ID_EXT)
+        {
+            OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, "File header not found",
+                "SkeletonSerializer::importSkeleton");
+        }
+
+        // Read version
+        String ver = readString(stream);
+        if ((ver != "[Serializer_v1.10]") &&
+            (ver != "[Serializer_v1.80]"))
+        {
+            OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR,
+                "Invalid file: version incompatible, file reports " + String(ver),
+                "SkeletonSerializer::importSkeleton");
+        }
+        mVersion = ver;
+
+        pushInnerChunk(stream);
+        unsigned short streamID = readChunk(stream);
 
-        unsigned short streamID;
         while(!stream->eof())
         {
-            streamID = readChunk(stream);
             switch (streamID)
             {
+            case SKELETON_BLENDMODE:
+            {
+                // Optional blend mode
+                uint16 blendMode;
+                readShorts(stream, &blendMode, 1);
+                pSkel->setBlendMode(static_cast<SkeletonAnimationBlendMode>(blendMode));
+                break;
+            }
             case SKELETON_BONE:
                 readBone(stream, pSkel);
                 break;
@@ -131,21 +149,41 @@ namespace Ogre {
                 break;
             case SKELETON_ANIMATION:
                 readAnimation(stream, pSkel);
-				break;
-			case SKELETON_ANIMATION_LINK:
-				readSkeletonAnimationLink(stream, pSkel);
-				break;
+                break;
+            case SKELETON_ANIMATION_LINK:
+                readSkeletonAnimationLink(stream, pSkel);
+                break;
+            default:
+                break;
             }
-        }
 
+            streamID = readChunk(stream);
+        }
         // Assume bones are stored in the binding pose
         pSkel->setBindingPose();
-
+        popInnerChunk(stream);
 
     }
+    
     //---------------------------------------------------------------------
-    void SkeletonSerializer::writeSkeleton(const Skeleton* pSkel)
+    void SkeletonSerializer::setWorkingVersion(SkeletonVersion ver)
     {
+        if (ver == SKELETON_VERSION_1_0)
+            mVersion = "[Serializer_v1.10]";
+        else mVersion = "[Serializer_v1.80]";
+    }
+    //---------------------------------------------------------------------
+    void SkeletonSerializer::writeSkeleton(const Skeleton* pSkel, SkeletonVersion ver)
+    {
+        
+        // Write blend mode
+        if ((int)ver > (int)SKELETON_VERSION_1_0)
+        {
+            writeChunkHeader(SKELETON_BLENDMODE, SSTREAM_OVERHEAD_SIZE + sizeof(unsigned short));
+            uint16 blendMode = static_cast<uint16>(pSkel->getBlendMode());
+            writeShorts(&blendMode, 1);
+        }
+        
         // Write each bone
         unsigned short numBones = pSkel->getNumBones();
         unsigned short i;
@@ -159,7 +197,7 @@ namespace Ogre {
         {
             Bone* pBone = pSkel->getBone(i);
             unsigned short handle = pBone->getHandle();
-            Bone* pParent = (Bone*)pBone->getParent(); 
+            Bone* pParent = static_cast<Bone*>(pBone->getParent());
             if (pParent != NULL) 
             {
                 writeBoneParent(pSkel, handle, pParent->getHandle());             
@@ -174,6 +212,10 @@ namespace Ogre {
         unsigned short handle = pBone->getHandle();
         // char* name
         writeString(pBone->getName());
+#if OGRE_SERIALIZER_VALIDATE_CHUNKSIZE
+        // Hack to fix chunk size validation:
+        mChunkSizeStack.back() += calcStringSize(pBone->getName());
+#endif
         // unsigned short handle            : handle of the bone, should be contiguous & start at 0
         writeShorts(&handle, 1);
         // Vector3 position                 : position of this bone relative to parent 
@@ -200,22 +242,45 @@ namespace Ogre {
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::writeAnimation(const Skeleton* pSkel, 
-        const Animation* anim)
+        const Animation* anim, SkeletonVersion ver)
     {
-        writeChunkHeader(SKELETON_ANIMATION, calcAnimationSize(pSkel, anim));
+        writeChunkHeader(SKELETON_ANIMATION, calcAnimationSize(pSkel, anim, ver));
 
         // char* name                       : Name of the animation
         writeString(anim->getName());
         // float length                      : Length of the animation in seconds
         float len = anim->getLength();
         writeFloats(&len, 1);
+        pushInnerChunk(mStream);
+        {
+        if ((int)ver > (int)SKELETON_VERSION_1_0)
+        {
+            if (anim->getUseBaseKeyFrame())
+            {
+                size_t size = SSTREAM_OVERHEAD_SIZE;
+                // char* baseAnimationName (including terminator)
+                    size += calcStringSize(anim->getBaseKeyFrameAnimationName());
+                // float baseKeyFrameTime
+                size += sizeof(float);
+                
+                writeChunkHeader(SKELETON_ANIMATION_BASEINFO, size);
+                
+                // char* baseAnimationName (blank for self)
+                writeString(anim->getBaseKeyFrameAnimationName());
+                
+                // float baseKeyFrameTime
+                float t = (float)anim->getBaseKeyFrameTime();
+                writeFloats(&t, 1);
+            }
+        }
 
         // Write all tracks
-        Animation::NodeTrackIterator trackIt = anim->getNodeTrackIterator();
-        while(trackIt.hasMoreElements())
+        for (const auto& it : anim->_getNodeTrackList())
         {
-            writeAnimationTrack(pSkel, trackIt.getNext());
+            writeAnimationTrack(pSkel, it.second);
         }
+        }
+        popInnerChunk(mStream);
 
     }
     //---------------------------------------------------------------------
@@ -225,24 +290,23 @@ namespace Ogre {
         writeChunkHeader(SKELETON_ANIMATION_TRACK, calcAnimationTrackSize(pSkel, track));
 
         // unsigned short boneIndex     : Index of bone to apply to
-        Bone* bone = (Bone*)track->getAssociatedNode();
+        Bone* bone = static_cast<Bone*>(track->getAssociatedNode());
         unsigned short boneid = bone->getHandle();
         writeShorts(&boneid, 1);
-
+        pushInnerChunk(mStream);
         // Write all keyframes
         for (unsigned short i = 0; i < track->getNumKeyFrames(); ++i)
         {
             writeKeyFrame(pSkel, track->getNodeKeyFrame(i));
         }
-
+        popInnerChunk(mStream);
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::writeKeyFrame(const Skeleton* pSkel, 
         const TransformKeyFrame* key)
     {
 
-        writeChunkHeader(SKELETON_ANIMATION_TRACK_KEYFRAME, 
-            calcKeyFrameSize(pSkel, key));
+        writeChunkHeader(SKELETON_ANIMATION_TRACK_KEYFRAME, calcKeyFrameSize(pSkel, key));
 
         // float time                    : The time position (seconds)
         float time = key->getTime();
@@ -261,16 +325,7 @@ namespace Ogre {
     size_t SkeletonSerializer::calcBoneSize(const Skeleton* pSkel, 
         const Bone* pBone)
     {
-        size_t size = STREAM_OVERHEAD_SIZE;
-
-        // handle
-        size += sizeof(unsigned short);
-
-        // position
-        size += sizeof(float) * 3;
-
-        // orientation
-        size += sizeof(float) * 4;
+        size_t size = calcBoneSizeWithoutScale(pSkel, pBone);
 
         // scale
         if (pBone->getScale() != Vector3::UNIT_SCALE)
@@ -284,7 +339,11 @@ namespace Ogre {
     size_t SkeletonSerializer::calcBoneSizeWithoutScale(const Skeleton* pSkel, 
         const Bone* pBone)
     {
-        size_t size = STREAM_OVERHEAD_SIZE;
+        size_t size = SSTREAM_OVERHEAD_SIZE;
+
+        // TODO: Add this for next skeleton format!
+        // Currently it is broken, because to determine that we have scale, it will compare chunk size.
+        //size += calcStringSize(pBone->getName());
 
         // handle
         size += sizeof(unsigned short);
@@ -300,7 +359,7 @@ namespace Ogre {
     //---------------------------------------------------------------------
     size_t SkeletonSerializer::calcBoneParentSize(const Skeleton* pSkel)
     {
-        size_t size = STREAM_OVERHEAD_SIZE;
+        size_t size = SSTREAM_OVERHEAD_SIZE;
 
         // handle
         size += sizeof(unsigned short);
@@ -311,21 +370,31 @@ namespace Ogre {
         return size;
     }
     //---------------------------------------------------------------------
-    size_t SkeletonSerializer::calcAnimationSize(const Skeleton* pSkel, 
-        const Animation* pAnim)
+    size_t SkeletonSerializer::calcAnimationSize(const Skeleton* pSkel, const Animation* pAnim, SkeletonVersion ver)
     {
-        size_t size = STREAM_OVERHEAD_SIZE;
+        size_t size = SSTREAM_OVERHEAD_SIZE;
 
         // Name, including terminator
-        size += pAnim->getName().length() + 1;
+        size += calcStringSize(pAnim->getName());
         // length
         size += sizeof(float);
 
+        if ((int)ver > (int)SKELETON_VERSION_1_0)
+        {
+            if (pAnim->getUseBaseKeyFrame())
+            {
+                size += SSTREAM_OVERHEAD_SIZE;
+                // char* baseAnimationName (including terminator)
+                size += calcStringSize(pAnim->getBaseKeyFrameAnimationName());
+                // float baseKeyFrameTime
+                size += sizeof(float);
+            }
+        }
+
         // Nested animation tracks
-		Animation::NodeTrackIterator trackIt = pAnim->getNodeTrackIterator();
-		while(trackIt.hasMoreElements())
-		{
-            size += calcAnimationTrackSize(pSkel, trackIt.getNext());
+        for (const auto& it : pAnim->_getNodeTrackList())
+        {
+            size += calcAnimationTrackSize(pSkel, it.second);
         }
 
         return size;
@@ -334,7 +403,7 @@ namespace Ogre {
     size_t SkeletonSerializer::calcAnimationTrackSize(const Skeleton* pSkel, 
         const NodeAnimationTrack* pTrack)
     {
-        size_t size = STREAM_OVERHEAD_SIZE;
+        size_t size = SSTREAM_OVERHEAD_SIZE;
 
         // unsigned short boneIndex     : Index of bone to apply to
         size += sizeof(unsigned short);
@@ -351,14 +420,8 @@ namespace Ogre {
     size_t SkeletonSerializer::calcKeyFrameSize(const Skeleton* pSkel, 
         const TransformKeyFrame* pKey)
     {
-        size_t size = STREAM_OVERHEAD_SIZE;
+        size_t size = calcKeyFrameSizeWithoutScale(pSkel, pKey);
 
-        // float time                    : The time position (seconds)
-        size += sizeof(float);
-        // Quaternion rotate            : Rotation to apply at this keyframe
-        size += sizeof(float) * 4;
-        // Vector3 translate            : Translation to apply at this keyframe
-        size += sizeof(float) * 3;
         // Vector3 scale                : Scale to apply at this keyframe
         if (pKey->getScale() != Vector3::UNIT_SCALE)
         {
@@ -371,7 +434,7 @@ namespace Ogre {
     size_t SkeletonSerializer::calcKeyFrameSizeWithoutScale(const Skeleton* pSkel, 
         const TransformKeyFrame* pKey)
     {
-        size_t size = STREAM_OVERHEAD_SIZE;
+        size_t size = SSTREAM_OVERHEAD_SIZE;
 
         // float time                    : The time position (seconds)
         size += sizeof(float);
@@ -402,6 +465,13 @@ namespace Ogre {
         Quaternion q;
         readObject(stream, q);
         pBone->setOrientation(q);
+
+#if OGRE_SERIALIZER_VALIDATE_CHUNKSIZE
+        // Hack to fix chunk size validation:
+        mChunkSizeStack.back() += calcStringSize(name);
+#endif
+        // TODO: don't depend on mCurrentstreamLen in next skeleton format!
+        // Currently we use wrong chunk sizes, but we can't fix it, because we depend on mCurrentstreamLen
         // Do we have scale?
         if (mCurrentstreamLen > calcBoneSizeWithoutScale(pSkel, pBone))
         {
@@ -409,6 +479,7 @@ namespace Ogre {
             readObject(stream, scale);
             pBone->setScale(scale);
         }
+        
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::readBoneParent(DataStreamPtr& stream, Skeleton* pSkel)
@@ -441,11 +512,29 @@ namespace Ogre {
         readFloats(stream, &len, 1);
 
         Animation *pAnim = pSkel->createAnimation(name, len);
-
         // Read all tracks
         if (!stream->eof())
         {
+            pushInnerChunk(stream);
             unsigned short streamID = readChunk(stream);
+            // Optional base info is possible
+            if (streamID == SKELETON_ANIMATION_BASEINFO)
+            {
+                // char baseAnimationName
+                String baseAnimName = readString(stream);
+                // float baseKeyFrameTime
+                float baseKeyTime;
+                readFloats(stream, &baseKeyTime, 1);
+                
+                pAnim->setUseBaseKeyFrame(true, baseKeyTime, baseAnimName);
+                
+                if (!stream->eof())
+                {
+                    // Get next stream
+                    streamID = readChunk(stream);
+                }
+            }
+            
             while(streamID == SKELETON_ANIMATION_TRACK && !stream->eof())
             {
                 readAnimationTrack(stream, pAnim, pSkel);
@@ -459,13 +548,10 @@ namespace Ogre {
             if (!stream->eof())
             {
                 // Backpedal back to start of this stream if we've found a non-track
-                stream->skip(-STREAM_OVERHEAD_SIZE);
+                backpedalChunkHeader(stream);
             }
-
+            popInnerChunk(stream);
         }
-
-
-
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::readAnimationTrack(DataStreamPtr& stream, Animation* anim, 
@@ -484,6 +570,7 @@ namespace Ogre {
         // Keep looking for nested keyframes
         if (!stream->eof())
         {
+            pushInnerChunk(stream);
             unsigned short streamID = readChunk(stream);
             while(streamID == SKELETON_ANIMATION_TRACK_KEYFRAME && !stream->eof())
             {
@@ -498,9 +585,9 @@ namespace Ogre {
             if (!stream->eof())
             {
                 // Backpedal back to start of this stream if we've found a non-keyframe
-                stream->skip(-STREAM_OVERHEAD_SIZE);
+                backpedalChunkHeader(stream);
             }
-
+            popInnerChunk(stream);
         }
 
 
@@ -531,46 +618,46 @@ namespace Ogre {
             kf->setScale(scale);
         }
     }
-	//---------------------------------------------------------------------
-	void SkeletonSerializer::writeSkeletonAnimationLink(const Skeleton* pSkel, 
-		const LinkedSkeletonAnimationSource& link)
-	{
-		writeChunkHeader(SKELETON_ANIMATION_LINK, 
-			calcSkeletonAnimationLinkSize(pSkel, link));
-
-		// char* skeletonName
-		writeString(link.skeletonName);
-		// float scale
-		writeFloats(&(link.scale), 1);
-
-	}
     //---------------------------------------------------------------------
-	size_t SkeletonSerializer::calcSkeletonAnimationLinkSize(const Skeleton* pSkel, 
-		const LinkedSkeletonAnimationSource& link)
-	{
-		size_t size = STREAM_OVERHEAD_SIZE;
+    void SkeletonSerializer::writeSkeletonAnimationLink(const Skeleton* pSkel, 
+        const LinkedSkeletonAnimationSource& link)
+    {
+        writeChunkHeader(SKELETON_ANIMATION_LINK, 
+            calcSkeletonAnimationLinkSize(pSkel, link));
 
-		// char* skeletonName
-		size += link.skeletonName.length() + 1;
-		// float scale
-		size += sizeof(float);
+        // char* skeletonName
+        writeString(link.skeletonName);
+        // float scale
+        writeFloats(&(link.scale), 1);
 
-		return size;
+    }
+    //---------------------------------------------------------------------
+    size_t SkeletonSerializer::calcSkeletonAnimationLinkSize(const Skeleton* pSkel, 
+        const LinkedSkeletonAnimationSource& link)
+    {
+        size_t size = SSTREAM_OVERHEAD_SIZE;
 
-	}
-	//---------------------------------------------------------------------
-	void SkeletonSerializer::readSkeletonAnimationLink(DataStreamPtr& stream, 
-		Skeleton* pSkel)
-	{
-		// char* skeletonName
-		String skelName = readString(stream);
-		// float scale
-		float scale;
-		readFloats(stream, &scale, 1);
+        // char* skeletonName
+        size += link.skeletonName.length() + 1;
+        // float scale
+        size += sizeof(float);
 
-		pSkel->addLinkedSkeletonAnimationSource(skelName, scale);
+        return size;
 
-	}
+    }
+    //---------------------------------------------------------------------
+    void SkeletonSerializer::readSkeletonAnimationLink(DataStreamPtr& stream, 
+        Skeleton* pSkel)
+    {
+        // char* skeletonName
+        String skelName = readString(stream);
+        // float scale
+        float scale;
+        readFloats(stream, &scale, 1);
+
+        pSkel->addLinkedSkeletonAnimationSource(skelName, scale);
+
+    }
 
 
 
