@@ -59,6 +59,8 @@ extern "C" {
 #include "OgreCommon.h"
 #include "OgreDataStream.h"
 #include "OgreDefaultWorkQueue.h"
+#include "OgreDepthBuffer.h"
+#include "OgreEntity.h"
 #include "OgreException.h"
 #include "OgreFileSystem.h"
 #include "OgreFrustum.h"
@@ -84,6 +86,8 @@ extern "C" {
 #include "OgreProfiler.h"
 #include "OgreQuaternion.h"
 #include "OgreRadixSort.h"
+#include "OgreRenderTarget.h"
+#include "OgreRenderTexture.h"
 #include "OgreRay.h"
 #include "OgreRectangle2D.h"
 #include "OgreBuiltinMovableFactories.h"
@@ -107,6 +111,7 @@ extern "C" {
 #include "OgreStringInterface.h"
 #include "OgreStringVector.h"
 #include "OgreSubMesh.h"
+#include "OgreSubEntity.h"
 #include "OgreTechnique.h"
 #include "OgreTextureManager.h"
 #include "Threading/OgreThreadHeaders.h"
@@ -132,10 +137,22 @@ extern "C" {
 #define OGRE_SERIALIZER_VALIDATE_CHUNKSIZE OGRE_DEBUG_MODE
 #endif
 
+#if OGRE_PROFILING == 1
+#define OgreGpuEventScope(name) GpuEventScope _gpuEventScope(name)
+#else
+#define OgreGpuEventScope(name)
+#endif
+
 namespace Ogre
 {
 void logMaterialNotFound(const String& name, const String& groupName, const String& destType, const String& destName,
                          LogMessageLevel lml = LML_CRITICAL);
+
+struct GpuEventScope
+{
+    GpuEventScope(const String& name) { Root::getSingleton().getRenderSystem()->beginProfileEvent(name); }
+    ~GpuEventScope() { Root::getSingleton().getRenderSystem()->endProfileEvent(); }
+};
 }
 
 #endif 
