@@ -27,14 +27,11 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 
-#include "OgreEntity.h"
-#include "OgreSubEntity.h"
-
 namespace Ogre {
     //-----------------------------------------------------------------------
     SubEntity::SubEntity (Entity* parent, SubMesh* subMeshBasis)
         : Renderable(), mParentEntity(parent),
-        mSubMesh(subMeshBasis), mMaterialLodIndex(0), mCachedCamera(0)
+        mSubMesh(subMeshBasis), mCachedCamera(0)
     {
         mVisible = true;
         mRenderQueueID = 0;
@@ -49,11 +46,6 @@ namespace Ogre {
     }
     SubEntity::~SubEntity() = default; // ensure unique_ptr destructors are in cpp
     //-----------------------------------------------------------------------
-    SubMesh* SubEntity::getSubMesh(void)
-    {
-        return mSubMesh;
-    }
-    //-----------------------------------------------------------------------
     const String& SubEntity::getMaterialName(void) const
     {
         return mMaterialPtr ? mMaterialPtr->getName() : BLANKSTRING;
@@ -91,11 +83,6 @@ namespace Ogre {
         mParentEntity->reevaluateVertexProcessing();
     }
     //-----------------------------------------------------------------------
-    Technique* SubEntity::getTechnique(void) const
-    {
-        return mMaterialPtr->getBestTechnique(mMaterialLodIndex, this);
-    }
-    //-----------------------------------------------------------------------
     void SubEntity::getRenderOperation(RenderOperation& op)
     {
         // Use LOD
@@ -111,28 +98,18 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void SubEntity::setIndexDataStartIndex(size_t start_index)
+    void SubEntity::setIndexDataStartIndex(uint32 start_index)
     {
         if(start_index < mSubMesh->indexData->indexCount)
             mIndexStart = start_index;
     }
     //-----------------------------------------------------------------------
-    size_t SubEntity::getIndexDataStartIndex() const
+    void SubEntity::setIndexDataEndIndex(uint32 end_index)
     {
-        return mIndexStart;
-    }
-    //-----------------------------------------------------------------------
-    void SubEntity::setIndexDataEndIndex(size_t end_index)
-    {
-        if(end_index > 0 && end_index <= mSubMesh->indexData->indexCount)
+        if(end_index <= mSubMesh->indexData->indexCount)
             mIndexEnd = end_index;
     }
     //-----------------------------------------------------------------------
-    size_t SubEntity::getIndexDataEndIndex() const
-    {
-        return mIndexEnd;
-    }
-    //-----------------------------------------------------------------------
     void SubEntity::resetIndexDataStartEndIndex()
     {
         mIndexStart = 0;
@@ -182,6 +159,11 @@ namespace Ogre {
                 mSubMesh->parent->sharedBlendIndexToBoneIndexMap : mSubMesh->blendIndexToBoneIndexMap;
             assert(indexMap.size() <= mParentEntity->mNumBoneMatrices);
 
+            if (MeshManager::getBonesUseObjectSpace())
+            {
+                *xform++ = mParentEntity->_getParentNodeFullTransform();
+            }
+
             if (mParentEntity->_isSkeletonAnimated())
             {
                 // Bones, use cached matrices built when Entity::_updateRenderQueue was called
@@ -189,11 +171,6 @@ namespace Ogre {
                                                                           : mParentEntity->mBoneWorldMatrices;
                 assert(boneMatrices);
 
-                if (MeshManager::getBonesUseObjectSpace())
-                {
-                    *xform++ = mParentEntity->_getParentNodeFullTransform();
-                }
-
                 for (auto idx : indexMap)
                 {
                     *xform++ = boneMatrices[idx];
@@ -201,8 +178,10 @@ namespace Ogre {
             }
             else
             {
+                auto& value = MeshManager::getBonesUseObjectSpace() ? Affine3::IDENTITY
+                                                                    : mParentEntity->_getParentNodeFullTransform();
                 // All animations disabled, use parent entity world transform only
-                std::fill_n(xform, indexMap.size(), mParentEntity->_getParentNodeFullTransform());
+                std::fill_n(xform, indexMap.size(), value);
             }
         }
     }
