@@ -65,7 +65,7 @@ namespace Ogre
 * %Codec for loading geometry using the [Open-Asset-Importer](https://github.com/assimp/assimp)
 * @{
 */
-class _OgreAssimpExport AssimpLoader
+class AssimpLoader
 {
 public:
     enum LoaderParams
@@ -102,15 +102,14 @@ private:
     bool _load(const char* name, Assimp::Importer& importer, Mesh* mesh, SkeletonPtr& skeletonPtr,
                const Options& options);
     bool createSubMesh(const String& name, int index, const aiNode* pNode, const aiMesh* mesh,
-                       const aiMaterial* mat, Mesh* mMesh, AxisAlignedBox& mAAB);
-    MaterialPtr createMaterial(const aiMaterial* mat, const Ogre::String &group);
+                       const MaterialPtr& matptr, Mesh* mMesh, AxisAlignedBox& mAAB);
     void grabNodeNamesFromNode(const aiScene* mScene, const aiNode* pNode);
     void grabBoneNamesFromNode(const aiScene* mScene, const aiNode* pNode);
     void computeNodesDerivedTransform(const aiScene* mScene, const aiNode* pNode,
                                       const aiMatrix4x4& accTransform);
     void createBonesFromNode(const aiScene* mScene, const aiNode* pNode);
     void createBoneHiearchy(const aiScene* mScene, const aiNode* pNode);
-    void loadDataFromNode(const aiScene* mScene, const aiNode* pNode, Mesh* mesh);
+    AxisAlignedBox loadDataFromNode(const aiScene* mScene, const aiNode* pNode, Mesh* mesh);
     void markAllChildNodesAsNeeded(const aiNode* pNode);
     void flagNodeAsNeeded(const char* name);
     bool isNodeNeeded(const char* name);
@@ -136,7 +135,6 @@ private:
     static int msBoneCount;
 
     bool mQuietMode;
-    bool mUseIndexBuffer;
     Real mTicksPerSecond;
     Real mAnimationSpeedModifier;
 };
