@@ -32,6 +32,8 @@ THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreAssimpLoader.h"
+#include "OgreString.h"
+#include "OgreStringConverter.h"
 
 #include <assimp/version.h>
 #include <assimp/scene.h>
@@ -360,18 +362,41 @@ bool AssimpLoader::load(const String& source, Mesh* mesh, SkeletonPtr& skeletonP
 bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mesh, SkeletonPtr& skeletonPtr,
                          const Options& options)
 {
+    mAnimationSpeedModifier = options.animationSpeedModifier;
+    mLoaderParams = options.params;
+    mCustomAnimationName = options.customAnimationName;
+
+    float maxEdgeAngle = options.maxEdgeAngle;
+    int postProcessSteps = options.postProcessSteps;
+    auto optsAny = mesh->getUserObjectBindings().getUserAny("_AssimpLoaderOptions");
+    if(optsAny.has_value())
+    {
+        auto strOpts = any_cast<BinaryOptionList>(optsAny);
+        if(strOpts.find("quiet") != strOpts.end())
+            mLoaderParams |= LP_QUIET_MODE;
+        if(strOpts.find("customAnimationName") != strOpts.end())
+            mCustomAnimationName = strOpts["customAnimationName"];
+        if(strOpts.find("animationSpeedModifier") != strOpts.end())
+            StringConverter::parse(strOpts["animationSpeedModifier"], mAnimationSpeedModifier);
+        if(strOpts.find("maxEdgeAngle") != strOpts.end())
+            StringConverter::parse(strOpts["maxEdgeAngle"], maxEdgeAngle);
+        if(strOpts.find("postProcessSteps") != strOpts.end())
+            StringConverter::parse(strOpts["postProcessSteps"], postProcessSteps);
+        if(strOpts.find("cutAnimation") != strOpts.end())
+            mLoaderParams |= LP_CUT_ANIMATION_WHERE_NO_FURTHER_CHANGE;
+    }
+
     uint32 flags = aiProcessPreset_TargetRealtime_Fast | aiProcess_TransformUVCoords | aiProcess_FlipUVs;
     flags &= ~(aiProcess_JoinIdenticalVertices | aiProcess_CalcTangentSpace); // optimize for fast loading
 
-    if(StringUtil::endsWith(name, ".gltf") || StringUtil::endsWith(name, ".glb"))
-        flags |= aiProcess_JoinIdenticalVertices;
-
-    flags |= options.postProcessSteps;
+    flags |= postProcessSteps;
 
     if((flags & (aiProcess_GenSmoothNormals | aiProcess_GenNormals)) != aiProcess_GenNormals)
         flags &= ~aiProcess_GenNormals; // prefer smooth normals
 
-    importer.SetPropertyFloat("PP_GSN_MAX_SMOOTHING_ANGLE", options.maxEdgeAngle);
+    importer.SetPropertyFloat("PP_GSN_MAX_SMOOTHING_ANGLE", maxEdgeAngle);
+    importer.SetPropertyBool(AI_CONFIG_IMPORT_FBX_EMBEDDED_TEXTURES_LEGACY_NAMING, true);
+
     const aiScene* scene = importer.ReadFile(name, flags);
 
     // If the import failed, report it
@@ -381,11 +406,7 @@ bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mes
         return false;
     }
 
-    mAnimationSpeedModifier = options.animationSpeedModifier;
-    mLoaderParams = options.params;
     mQuietMode = mLoaderParams & LP_QUIET_MODE;
-    mUseIndexBuffer = flags & aiProcess_JoinIdenticalVertices;
-    mCustomAnimationName = options.customAnimationName;
     mNodeDerivedTransformByName.clear();
 
     String basename, extension;
@@ -419,7 +440,7 @@ bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mes
     {
         const aiTexture* tex = scene->mTextures[i];
         auto texname =
-            StringUtil::format("%s%s.%s", mesh->getName().c_str(), tex->mFilename.C_Str(), tex->achFormatHint);
+            StringUtil::format("%s%s%d.%s", mesh->getName().c_str(), tex->mFilename.C_Str(), i, tex->achFormatHint);
         if (TextureManager::getSingleton().resourceExists(texname, mesh->getGroup()))
             continue;
 
@@ -445,7 +466,10 @@ bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mes
         TextureManager::getSingleton().loadImage(texname, mesh->getGroup(), img);
     }
 
-    loadDataFromNode(scene, scene->mRootNode, mesh);
+    auto aabb = loadDataFromNode(scene, scene->mRootNode, mesh);
+    // We must indicate the bounding box
+    mesh->_setBounds(aabb);
+    mesh->_setBoundingSphereRadius((aabb.getMaximum() - aabb.getMinimum()).length() / 2);
 
     Assimp::DefaultLogger::kill();
 
@@ -608,7 +632,7 @@ void AssimpLoader::parseAnimation(const aiScene* mScene, int index, aiAnimation*
             defBonePoseInv.makeInverseTransform(bone->getPosition(), bone->getScale(),
                                                 bone->getOrientation());
 
-            NodeAnimationTrack* track = animation->createNodeTrack(i, bone);
+            NodeAnimationTrack* track = animation->createNodeTrack(bone->getHandle(), bone);
 
             // Ogre needs translate rotate and scale for each keyframe in the track
             KeyframesMap keyframes;
@@ -892,7 +916,8 @@ static bool getTextureName(const aiMaterial* mat, aiTextureType type, const aiSc
         const aiTexture* tex = scene->GetEmbeddedTexture(path.C_Str());
         if(tex)
         {
-            basename = StringUtil::format("%s%s.%.8s", meshName.c_str(), tex->mFilename.C_Str(), tex->achFormatHint);
+            basename = StringUtil::format("%s%s%s.%.8s", meshName.c_str(), tex->mFilename.C_Str(), path.C_Str() + 1,
+                                          tex->achFormatHint);
             return true;
         }
 
@@ -1094,9 +1119,11 @@ bool AssimpLoader::createSubMesh(const String& name, int index, const aiNode* pN
     aiVector3D* tang = mesh->mTangents;
     aiColor4D *col = mesh->mColors[0];
 
+    if(!norm)
+        matptr->setLightingEnabled(false);
+
     // We must create the vertex data, indicating how many vertices there will be
-    submesh->useSharedVertices = false;
-    submesh->vertexData = new VertexData();
+    submesh->createVertexData();
     submesh->vertexData->vertexStart = 0;
     submesh->vertexData->vertexCount = mesh->mNumVertices;
 
@@ -1265,9 +1292,6 @@ bool AssimpLoader::createSubMesh(const String& name, int index, const aiNode* pN
         LogManager::getSingleton().logMessage(StringConverter::toString(mesh->mNumFaces) + " faces");
     }
 
-    if(!mUseIndexBuffer)
-        return true;
-
     aiFace* faces = mesh->mFaces;
     int faceSz = mesh->mPrimitiveTypes == aiPrimitiveType_LINE ? 2 : 3;
 
@@ -1315,12 +1339,11 @@ bool AssimpLoader::createSubMesh(const String& name, int index, const aiNode* pN
     return true;
 }
 
-void AssimpLoader::loadDataFromNode(const aiScene* mScene, const aiNode* pNode, Mesh* mesh)
+AxisAlignedBox AssimpLoader::loadDataFromNode(const aiScene* mScene, const aiNode* pNode, Mesh* mesh)
 {
+    AxisAlignedBox aabb;
     if (pNode->mNumMeshes > 0)
     {
-        AxisAlignedBox mAAB = mesh->getBounds();
-
         for (unsigned int idx = 0; idx < pNode->mNumMeshes; ++idx)
         {
             aiMesh* pAIMesh = mScene->mMeshes[pNode->mMeshes[idx]];
@@ -1333,20 +1356,18 @@ void AssimpLoader::loadDataFromNode(const aiScene* mScene, const aiNode* pNode,
             // Create a material instance for the mesh.
             const aiMaterial* pAIMaterial = mScene->mMaterials[pAIMesh->mMaterialIndex];
             MaterialPtr matptr = createMaterial(pAIMaterial, mesh->getGroup(), mesh->getName(), mScene, !mQuietMode);
-            createSubMesh(pNode->mName.data, idx, pNode, pAIMesh, matptr, mesh, mAAB);
+            createSubMesh(pNode->mName.data, idx, pNode, pAIMesh, matptr, mesh, aabb);
         }
-
-        // We must indicate the bounding box
-        mesh->_setBounds(mAAB);
-        mesh->_setBoundingSphereRadius((mAAB.getMaximum() - mAAB.getMinimum()).length() / 2);
     }
 
     // Traverse all child nodes of the current node instance
     for (unsigned int childIdx = 0; childIdx < pNode->mNumChildren; childIdx++)
     {
         const aiNode* pChildNode = pNode->mChildren[childIdx];
-        loadDataFromNode(mScene, pChildNode, mesh);
+        aabb.merge(loadDataFromNode(mScene, pChildNode, mesh));
     }
+
+    return aabb;
 }
 
 static std::vector<std::unique_ptr<Codec> > registeredCodecs;
@@ -1371,7 +1392,7 @@ struct AssimpCodec : public Codec
 
     static void startup()
     {
-        String version = StringUtil::format("Assimp - %d.%d.%d - Open-Asset-Importer", aiGetVersionMajor(),
+        String version = StringUtil::format("Assimp - %d.%d.%x - Open-Asset-Importer", aiGetVersionMajor(),
                                             aiGetVersionMinor(), aiGetVersionRevision());
         LogManager::getSingleton().logMessage(version);
 
@@ -1431,4 +1452,4 @@ extern "C" void _OgreAssimpExport dllStopPlugin()
     AssimpCodec::shutdown();
 }
 #endif
-} // namespace Ogre
\ No newline at end of file
+} // namespace Ogre
