@@ -74,6 +74,15 @@ namespace Ogre
         }
         mPortals.clear();
 
+        AntiPortal * ap;
+        AntiPortalList::iterator iap = mAntiPortals.begin();
+        for (iap = mAntiPortals.begin(); iap != mAntiPortals.end(); iap++)
+        {
+            ap = *iap;
+            OGRE_DELETE ap;
+        }
+        mAntiPortals.clear();
+
         // delete all the zones
         for (ZoneMap::iterator j = mZones.begin();
             j != mZones.end(); ++j)
@@ -103,6 +112,15 @@ namespace Ogre
         }
         mPortals.clear();
 
+        AntiPortal * ap;
+        AntiPortalList::iterator iap = mAntiPortals.begin();
+        for (iap = mAntiPortals.begin(); iap != mAntiPortals.end(); iap++)
+        {
+            ap = *iap;
+            OGRE_DELETE ap;
+        }
+        mAntiPortals.clear();
+
         // delete all the zones
         for (ZoneMap::iterator j = mZones.begin();
             j != mZones.end(); ++j)
@@ -441,23 +459,6 @@ namespace Ogre
         SceneManager::_renderScene(cam, vp, includeOverlays);
     }
 
-    /* enable/disable sky rendering */
-    void PCZSceneManager::enableSky(bool onoff)
-    {
-        if (getSkyBoxNode())
-        {
-            setSkyBoxEnabled(onoff);
-        }
-        else if (getSkyDomeNode())
-        {
-            setSkyDomeEnabled(onoff);
-        }
-        else if (getSkyPlaneNode())
-        {
-            setSkyPlaneEnabled(onoff);
-        }
-    }
-
     /* Set the zone which contains the sky node */
     void PCZSceneManager::setSkyZone(PCZone * zone)
     {
@@ -466,25 +467,12 @@ namespace Ogre
             // if no zone specified, use default zone
             zone = mDefaultZone;
         }
-        if (auto node = (PCZSceneNode*)getSkyBoxNode())
+        if (auto node = (PCZSceneNode*)getSkyNode())
         {
             node->setHomeZone(zone);
             node->anchorToHomeZone(zone);
             zone->setHasSky(true);
         }
-        if (auto node = (PCZSceneNode*)getSkyDomeNode())
-        {
-            node->setHomeZone(zone);
-            node->anchorToHomeZone(zone);
-            zone->setHasSky(true);
-        }
-        if (auto node = (PCZSceneNode*)getSkyPlaneNode())
-        {
-            node->setHomeZone(zone);
-            node->anchorToHomeZone(zone);
-            zone->setHasSky(true);
-        }
-        
     }
 
     //-----------------------------------------------------------------------
@@ -878,24 +866,6 @@ namespace Ogre
         pczsn->setHomeZone(zone);
     }
 
-    // (optional) post processing for any scene node found visible for the frame
-    void PCZSceneManager::_alertVisibleObjects( void )
-    {
-        OGRE_EXCEPT( Exception::ERR_NOT_IMPLEMENTED,
-            "Function doesn't do as advertised",
-            "PCZSceneManager::_alertVisibleObjects" );
-
-//        NodeList::iterator it = mVisible.begin();
-//
-//        while ( it != mVisible.end() )
-//        {
-//            SceneNode * node = *it;
-//            // this is where you would do whatever you wanted to the visible node
-//            // but right now, it does nothing.
-//            ++it;
-//        }
-    }
-
     //-----------------------------------------------------------------------
     Light* PCZSceneManager::createLight(const String& name)
     {
