@@ -51,16 +51,7 @@ namespace Ogre
     //---------------------------------------------------------------------
     D3D11Texture::~D3D11Texture()
     {
-        // have to call this here reather than in Resource destructor
-        // since calling virtual methods in base destructors causes crash
-        if (isLoaded())
-        {
-            unload(); 
-        }
-        else
-        {
-            freeInternalResources();
-        }
+        unload();
     }
     //---------------------------------------------------------------------
     void D3D11Texture::notifyDeviceLost(D3D11Device* device)
@@ -114,18 +105,6 @@ namespace Ogre
 
     }
     //---------------------------------------------------------------------
-    void D3D11Texture::loadImpl()
-    {
-        Texture::loadImpl();
-
-        if (mUsage & TU_RENDERTARGET)
-        {
-            return;
-        }
-
-        _setSrcAttributes(mWidth, mHeight, mDepth, mFormat);
-    }
-    //---------------------------------------------------------------------
     void D3D11Texture::freeInternalResourcesImpl()
     {
         mpTex.Reset();
@@ -158,7 +137,7 @@ namespace Ogre
             D3D11RenderSystem* rsys = static_cast<D3D11RenderSystem*>(Root::getSingleton().getRenderSystem());
             // http://msdn.microsoft.com/en-us/library/windows/desktop/ff476150%28v=vs.85%29.aspx#ID3D11Device_CreateTexture2D
             // 10Level9, When using D3D11_BIND_SHADER_RESOURCE, SampleDesc.Count must be 1.
-            if(rsys->_getFeatureLevel() >= D3D_FEATURE_LEVEL_10_0 || (mUsage & TU_NOT_SRV))
+            if(rsys->_getFeatureLevel() >= D3D_FEATURE_LEVEL_10_0 || (mUsage & TU_NOT_SAMPLED))
                 rsys->determineFSAASettings(mFSAA, mFSAAHint, mD3DFormat, &mFSAAType);
         }
 
@@ -183,7 +162,7 @@ namespace Ogre
             this->_create3DTex();
             break;
         default:
-            this->freeInternalResources();
+            this->unloadImpl();
             OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Unknown texture type", "D3D11Texture::createInternalResources" );
         }
     }
@@ -214,7 +193,7 @@ namespace Ogre
         // check result and except if failed
         if (FAILED(hr) || mDevice.isError())
         {
-            this->freeInternalResources();
+            this->unloadImpl();
 			String errorDescription = mDevice.getErrorDescription(hr);
 			OGRE_EXCEPT_EX(Exception::ERR_RENDERINGAPI_ERROR, hr,
 				"Error creating texture\nError Description:" + errorDescription,
@@ -312,7 +291,7 @@ namespace Ogre
         // check result and except if failed
         if (FAILED(hr) || mDevice.isError())
         {
-            this->freeInternalResources();
+            this->unloadImpl();
             String errorDescription = mDevice.getErrorDescription(hr);
 			OGRE_EXCEPT_EX(Exception::ERR_RENDERINGAPI_ERROR, hr,
                 "Error creating texture\nError Description:" + errorDescription, 
@@ -431,7 +410,7 @@ namespace Ogre
         // check result and except if failed
         if (FAILED(hr) || mDevice.isError())
         {
-            this->freeInternalResources();
+            this->unloadImpl();
             String errorDescription = mDevice.getErrorDescription(hr);
 			OGRE_EXCEPT_EX(Exception::ERR_RENDERINGAPI_ERROR, hr,
                 "Error creating texture\nError Description:" + errorDescription, 
@@ -492,16 +471,6 @@ namespace Ogre
         _createSurfaceList();
     }
     //---------------------------------------------------------------------
-    void D3D11Texture::_setSrcAttributes(unsigned long width, unsigned long height, 
-        unsigned long depth, PixelFormat format)
-    { 
-        // set source image attributes
-        mSrcWidth = width; 
-        mSrcHeight = height; 
-        mSrcDepth = depth;
-        mSrcFormat = format;
-    }
-    //---------------------------------------------------------------------
     void D3D11Texture::_createSurfaceList(void)
     {
         // Create new list of surfaces
