@@ -67,13 +67,17 @@ namespace Ogre
             }
         }
 
+            #define GL_MESH_SHADER_BIT_NV 0x00000040
+            #define GL_TASK_SHADER_BIT_NV 0x00000080
             GLenum ogre2gltype[GPT_COUNT] = {
                 GL_VERTEX_SHADER_BIT,
                 GL_FRAGMENT_SHADER_BIT,
                 GL_GEOMETRY_SHADER_BIT,
                 GL_TESS_EVALUATION_SHADER_BIT,
                 GL_TESS_CONTROL_SHADER_BIT,
-                GL_COMPUTE_SHADER_BIT
+                GL_MESH_SHADER_BIT_NV,
+                GL_COMPUTE_SHADER_BIT,
+                GL_TASK_SHADER_BIT_NV
             };
 
             for (auto s : mShaders)
@@ -116,14 +120,7 @@ namespace Ogre
         GLuint progID = mShaders[fromProgType]->getGLProgramHandle();
         GLUniformCache* uniformCache = mShaders[fromProgType]->getUniformCache();
 
-        bool usesUBO = false;
-        if(const auto& ubo = static_cast<GLSLShader*>(mShaders[fromProgType])->getDefaultBuffer())
-        {
-            // we ignore ma
-            ubo->writeData(0, ubo->getSizeInBytes(), params->getConstantList().data(), true);
-            static_cast<GL3PlusHardwareBuffer*>(ubo.get())->bind();
-            usesUBO = true;
-        }
+        bool usesUBO = !params->hasLogicalIndexedParameters();
 
         // Iterate through uniform reference list and update uniform values
         for (const auto& it : params->getConstantDefinitions().map)
