@@ -268,8 +268,7 @@ namespace Ogre {
             mSliceTRT.reserve(mDepth);
             for(uint32 zoffset=0; zoffset<mDepth; ++zoffset)
             {
-                String name;
-                name = "rtt/" + StringConverter::toString((size_t)this) + "/" + baseName;
+                String name = getNameForRenderTexture(baseName);
                 RenderTexture *trt = OGRE_NEW MetalRenderTexture( device, name, this,
                                                                   renderTexture, resolveTexture,
                                                                   mFormat, zoffset, mFace, fsaa, level,
