@@ -109,9 +109,8 @@ public:
         schemRenderState->addTemplateSubRenderState(subRenderState);
 #endif
         mSceneMgr->setShadowTechnique(SHADOWTYPE_TEXTURE_MODULATIVE_INTEGRATED);
-        mSceneMgr->setShadowTextureSettings(256, 1, PF_FLOAT32_R);
+        mSceneMgr->setShadowTextureSettings(256, 1, PF_DEPTH16);
         mSceneMgr->setShadowTextureSelfShadow( true );
-        mSceneMgr->setShadowTextureCasterMaterial(MaterialManager::getSingleton().getByName("PSSM/shadow_caster"));
 
         auto statusPanel = mTrayMgr->createParamsPanel(TL_TOPLEFT, "HelpMessage", 200, {"Help"});
         statusPanel->setParamValue("Help", "H / F1");
@@ -153,8 +152,7 @@ public:
         mBillboardSet->setCastShadows(false);
         mLightCameraSN->attachObject(mBillboardSet);
 
-        // Creating a RRT for depth/shadow map
-        createLightCameraRTT(mLight0);
+        getLightCamera();
 
         // Create a rush of billboards according to the frustum of the camera(mLightCamera)
         // After it, we can use the lightcamera/billboards scenenode like a light projector
@@ -180,6 +178,8 @@ public:
 
     bool createLightShafts(BillboardSet* billboard, Camera* LightCamera, const int& NumberOfPlanes)
     {
+        mViewport->update(); // make sure LightCamera is synced to scene settings
+
         // Calculate the distance between planes
         float DistanceBetweenPlanes =
             (LightCamera->getFarClipDistance() - LightCamera->getNearClipDistance()) / NumberOfPlanes;
@@ -210,7 +210,7 @@ public:
         return true;
     }
 
-    void createLightCameraRTT(Light* light)
+    void getLightCamera()
     {
         // Create a texture for use as rtt
         TexturePtr LightCameraRTT = mSceneMgr->getShadowTexture(0);
@@ -219,9 +219,6 @@ public:
         Viewport* RT_Texture_Viewport = RT_Texture->getViewport(0);
         mLightCamera = RT_Texture_Viewport->getCamera();
 
-        mSceneMgr->getShadowCameraSetup()->getShadowCamera(mSceneMgr, mCamera, RT_Texture_Viewport, light, mLightCamera,
-                                                           0);
-
         mLightCamera->setDebugDisplayEnabled(true);
     }
 
