@@ -29,20 +29,7 @@ public:
 
     void testCapabilities(const RenderSystemCapabilities* caps) override
     {
-        if (!caps->hasCapability(RSC_TESSELLATION_HULL_PROGRAM) || !caps->hasCapability(RSC_TESSELLATION_DOMAIN_PROGRAM))
-        {
-            OGRE_EXCEPT(Exception::ERR_INVALID_STATE, "Your graphics card does not support tessellation shaders. Sorry!",
-                "Sample_PNTrianglesTessellation:testCapabilities");
-        }
-        if (!GpuProgramManager::getSingleton().isSyntaxSupported("vs_5_0") &&
-            !GpuProgramManager::getSingleton().isSyntaxSupported("hs_5_0") &&
-            !GpuProgramManager::getSingleton().isSyntaxSupported("ds_5_0") &&
-            !GpuProgramManager::getSingleton().isSyntaxSupported("ps_5_0") &&
-            !GpuProgramManager::getSingleton().isSyntaxSupported("hlsl"))
-        {
-            OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED, "Your card does not support the shader model 5.0 needed for this sample, "
-                "so you cannot run this sample. Sorry!", "Sample_PNTrianglesTessellation::testCapabilities");
-        }
+        requireMaterial("Ogre/SimpleTessellation");
     }
 
     bool frameRenderingQueued(const FrameEvent& evt) override
