@@ -550,11 +550,12 @@ protected:
     {
         auto matProfile = static_cast<TerrainMaterialGeneratorA::SM2Profile*>(
             mTerrainGlobals->getDefaultMaterialGenerator()->getActiveProfile());
+        matProfile->setReceiveDynamicShadowsEnabled(false); // force regen if colour/ depth shadows change
         matProfile->setReceiveDynamicShadowsEnabled(enabled);
         matProfile->setReceiveDynamicShadowsLowLod(SHADOWS_IN_LOW_LOD_MATERIAL);
 
         RTShader::RenderState* schemRenderState = mShaderGenerator->getRenderState(MSN_SHADERGEN);
-        if (auto srs = schemRenderState->getSubRenderState(RTShader::SRS_INTEGRATED_PSSM3))
+        if (auto srs = schemRenderState->getSubRenderState(RTShader::SRS_SHADOW_MAPPING))
         {
             schemRenderState->removeSubRenderState(srs);
         }
@@ -591,13 +592,10 @@ protected:
                 mSceneMgr->setShadowTextureSelfShadow(true);
                 mSceneMgr->setShadowCasterRenderBackFaces(true);
 
-                auto subRenderState = mShaderGenerator->createSubRenderState(RTShader::SRS_INTEGRATED_PSSM3);
+                auto subRenderState = mShaderGenerator->createSubRenderState(RTShader::SRS_SHADOW_MAPPING);
                 subRenderState->setParameter("split_points",
                                              static_cast<PSSMShadowCameraSetup*>(mPSSMSetup.get())->getSplitPoints());
                 schemRenderState->addTemplateSubRenderState(subRenderState);
-
-                mSceneMgr->setShadowTextureCasterMaterial(
-                    MaterialManager::getSingleton().getByName("PSSM/shadow_caster"));
             }
             else
             {
@@ -607,7 +605,6 @@ protected:
                 mSceneMgr->setShadowTextureConfig(2, 1024, 1024, PF_X8B8G8R8);
                 mSceneMgr->setShadowTextureSelfShadow(false);
                 mSceneMgr->setShadowCasterRenderBackFaces(false);
-                mSceneMgr->setShadowTextureCasterMaterial(MaterialPtr());
             }
 
             matProfile->setReceiveDynamicShadowsPSSM(static_cast<PSSMShadowCameraSetup*>(mPSSMSetup.get()));
